% ds1000zVisaAddrStr -- DS1000Z VISA Name
% read_wave_length -- read wave data length, relate to memory depth
% wave data written to the DS1000Z_Waveform.csv  
function DS1000Z_Waveform(ds1000zVisaAddrStr, read_wave_length)
ascTable = zeros(1, 256);

function buildAscTable(dev_object)
    % build asc to view value table
    ds1000z = dev_object;
    mYRef = 0;
    mYor = 0;
    mYInc = 0;
    fprintf(DS1000Z, ':WAVeform:YREFerence?');
    [mYRef] = fscanf(ds1000z, '%f', 100);
    fprintf(DS1000Z, ':WAVeform:YORigin?');
    [mYor] = fscanf(ds1000z, '%f', 100);
    fprintf(DS1000Z, ':WAVeform:YINCrement?');
    [mYInc] = fscanf(ds1000z, '%f', 100);
    
    for i = 1:256
        ascTable(i) = ((i - mYRef - mYor) * mYInc);
    end
end

% set read package size, must be < 1MBytes
if(read_wave_length > 250000)
    readPackageSize = 250000;
else
    readPackageSize = read_wave_length;
end
DS1000Z = visa('ni', ds1000zVisaAddrStr);
DS1000Z.InputBufferSize = (readPackageSize + 100);

try
    fopen(DS1000Z);
    total_len = 1;
    startReadPosition = 1;
    endReadPosition = 1;
    % build asc to view value table
    buildAscTable(DS1000Z);
    fprintf(DS1000Z, ':WAVeform:SOURce %s', 'CHAN1');
    fprintf(DS1000Z, ':WAVeform:MODE %s', 'RAW');
    fprintf(DS1000Z, ':WAVeform:FORMat %s', 'BYTE');
    fprintf(DS1000Z, ':STOP');
    while(1)
        % read wave data by package
        % set wave start position
        fprintf(DS1000Z, ':WAVeform:STARt %d', startReadPosition);
        fprintf(DS1000Z, 'WAVeform:STARt?');
        %[readedPosition, len] = fread(DS1000Z, 20480);
        [readedPosition, len] = fscanf(DS1000Z, '%d', 100);
        if(readedPosition ~= startReadPosition)
            break;
        end
        % set wave stop position
        endReadPosition = startReadPosition + readPackageSize - 1;
        fprintf(DS1000Z, ':WAVeform:STOP %d', endReadPosition);
        fprintf(DS1000Z, 'WAVeform:STOP?');
        [readedPosition, len] = fscanf(DS1000Z, '%d', 100);
        if(readedPosition ~= endReadPosition)
            break;
        end
        % read wave data
        fprintf(DS1000Z, ':wav:data?');
        [data,len] = fread(DS1000Z, (readPackageSize + 100));
        % reconstruct wave data
        waveData(total_len:total_len + (len - 13)) = data(12:len-1);
        total_len = total_len + (len - 13); 
        startReadPosition = endReadPosition + 1;
    end
catch err
    fclose(DS1000Z);
    delete(DS1000Z);
    clear DS1000Z;
end
    filePath = [mfilename('fullpath'), '.csv'];
    try 
        fd = fopen(filePath, 'w+');
        waveDataTemp = zeros(1, total_len);
        for i = 1 : total_len
            waveDataTemp(i) = ascTable(waveData(i));
            % write wave data to csv file
            fprintf(fd, '%s,\r\n', waveDataTemp(i));
        end
        fclose(fd);
    catch
        fclose(fd);
    end
fclose(DS1000Z);
delete(DS1000Z);
clear DS1000Z;
%waveDataRet = waveDataTemp;
end